/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.apply.operationSiteApply.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.flow.core.entity.BladeFlow;
import org.springblade.flow.engine.service.impl.FlowEngineServiceImpl;
import org.springblade.modules.apply.operationSiteApply.pojo.dto.OperationSiteApplyDTO;
import org.springblade.modules.apply.operationSiteApply.pojo.vo.ApplyVo;
import org.springblade.modules.flowable.examineInfo.service.IExamineInfoService;
import org.springblade.modules.flowable.processForm.pojo.entity.ProcessDefAuditStepEntity;
import org.springblade.modules.flowable.processForm.pojo.vo.ProcessDefAuditStepVO;
import org.springblade.modules.flowable.processForm.service.impl.ProcessDefAuditStepServiceImpl;
import org.springblade.modules.flowable.processForm.utils.FormUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.apply.operationSiteApply.pojo.entity.OperationSiteApplyEntity;
import org.springblade.modules.apply.operationSiteApply.pojo.vo.OperationSiteApplyVO;
import org.springblade.modules.apply.operationSiteApply.excel.OperationSiteApplyExcel;
import org.springblade.modules.apply.operationSiteApply.wrapper.OperationSiteApplyWrapper;
import org.springblade.modules.apply.operationSiteApply.service.IOperationSiteApplyService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 运维商准入申请表 控制器
 *
 * @author lzp
 * @since 2024-08-09
 */
@RestController
@AllArgsConstructor
@RequestMapping("blade-operationSiteApply/operationSiteApply")
@Tag(name = "运维商准入申请表", description = "运维商准入申请表接口")
public class OperationSiteApplyController extends BladeController {

	private final IOperationSiteApplyService operationSiteApplyService;

	private final IExamineInfoService examineInfoService;

	@Autowired
	private FlowEngineServiceImpl flowEngineService;

	/**
	 * 运维商准入申请表 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入operationSiteApply")
	public R<OperationSiteApplyVO> detail(OperationSiteApplyEntity operationSiteApply) {
		OperationSiteApplyEntity detail = operationSiteApplyService.getOne(Condition.getQueryWrapper(operationSiteApply));
		OperationSiteApplyVO vo = OperationSiteApplyWrapper.build().entityVO(detail);
			vo.setExamineInfoList(examineInfoService.findExamineInfo(detail.getId(), FormUtils.AGENT_ADMISSION_APPLICATION));
		List<ProcessDefAuditStepVO> flow = flowEngineService.findBladeFlowList(detail.getProcessInstanceId(),detail.getProcessDefId());
			vo.setBladeFlowList(flow);
			return R.data(vo);
	}
	/**
	 * 运维商准入申请表 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入operationSiteApply")
	public R<IPage<OperationSiteApplyVO>> page(OperationSiteApplyDTO req, Query query) {
		IPage<OperationSiteApplyVO> pages = operationSiteApplyService.selectOperationSiteApplyPage(Condition.getPage(query), req);
		return R.data(pages);
	}

	/**
	 * 运维商准入申请表 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增/编辑", description  = "传入operationSiteApply")
	public R save(@Valid @RequestBody OperationSiteApplyEntity operationSiteApply) {
		return R.status(operationSiteApplyService.saveApply(operationSiteApply));
	}

/*	*//**
	 * 运维商准入申请表 修改
	 *//*
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入operationSiteApply")
	public R update(@Valid @RequestBody OperationSiteApplyEntity operationSiteApply) {
		return R.status(operationSiteApplyService.updateById(operationSiteApply));
	}*/

	/**
	 * 运维商准入申请表 新增或修改
	 */
	@PostMapping("/examineSiteApply")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "审核通过", description  = "传入")
	public R examineSiteApply(@Valid @RequestBody  ApplyVo vo) {
		return R.status(operationSiteApplyService.examineSiteApply(vo.getId(), vo.getTaskId(), vo.getContent()));
	}

	/**
	 * 删除
	*/
	@GetMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "逻辑删除", description  = "传入id")
	public R remove(@Parameter(name = "id",description = "主键id",required = true) @RequestParam String id) {
		return R.status(operationSiteApplyService.deleteSiteApply(id));
	}

	//审核驳回
	@PostMapping("/rejectTask")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "审核驳回", description  = "传入")
	public R rejectTask(@Valid @RequestBody ApplyVo vo){

	return R.status(operationSiteApplyService.rejectTask(vo.getId(), vo.getTaskId(), vo.getContent()));
	}



	/**
	 * 导出数据
	 */
	@GetMapping("/export-operationSiteApply")
	@ApiOperationSupport(order = 9)
	@Operation(summary = "导出数据", description  = "传入operationSiteApply")
	public void exportOperationSiteApply(@Parameter(hidden = true) @RequestParam Map<String, Object> operationSiteApply, BladeUser bladeUser, HttpServletResponse response) {
		QueryWrapper<OperationSiteApplyEntity> queryWrapper = Condition.getQueryWrapper(operationSiteApply, OperationSiteApplyEntity.class);

		List<OperationSiteApplyExcel> list = operationSiteApplyService.exportOperationSiteApply(queryWrapper);
		ExcelUtil.export(response, "运维商准入申请表数据" + DateUtil.time(), "运维商准入申请表数据表", list, OperationSiteApplyExcel.class);
	}

}
