/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.material.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.system.service.IUserService;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.material.pojo.entity.MaterialOutboundEntity;
import org.springblade.modules.material.pojo.vo.MaterialOutboundVO;
import org.springblade.modules.material.excel.MaterialOutboundExcel;
import org.springblade.modules.material.wrapper.MaterialOutboundWrapper;
import org.springblade.modules.material.service.IMaterialOutboundService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 出库明细 控制器
 *
 * @author cswang
 * @since 2024-08-25
 */
@RestController
@AllArgsConstructor
@RequestMapping("wl/materialOutbound")
@Tag(name = "出库明细", description = "出库明细接口")
public class MaterialOutboundController extends BladeController {

	private final IMaterialOutboundService materialOutboundService;
	private final IUserService userService;

	/**
	 * 出库明细 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入materialOutbound")
	public R<MaterialOutboundVO> detail(MaterialOutboundEntity materialOutbound) {
		MaterialOutboundEntity detail = materialOutboundService.getOne(Condition.getQueryWrapper(materialOutbound));
		return R.data(MaterialOutboundWrapper.build().entityVO(detail));
	}
	/**
	 * 出库明细 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "分页", description  = "传入materialOutbound")
	public R<IPage<MaterialOutboundVO>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> materialOutbound, Query query) {
		query.setDescs("id");
		QueryWrapper<MaterialOutboundEntity> queryWrapper = Condition.getQueryWrapper(materialOutbound, MaterialOutboundEntity.class);
		queryWrapper.lambda().eq(MaterialOutboundEntity :: getKeepType,"3");
		IPage<MaterialOutboundEntity> pages = materialOutboundService.page(Condition.getPage(query),queryWrapper);
		IPage<MaterialOutboundVO> pageVO = MaterialOutboundWrapper.build().pageVO(pages);
		for(MaterialOutboundVO vo : pageVO.getRecords()){
			vo.setCreateName(userService.getById(vo.getCreateUser()).getName());
			if(vo.getAllowUser()!=null){
				vo.setAllowName(userService.getById(vo.getAllowUser()).getName());
			}
		}
		return R.data(pageVO);
	}

	/**
	 * 出库明细 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入materialOutbound")
	public R<IPage<MaterialOutboundVO>> page(MaterialOutboundVO materialOutbound, Query query) {
		IPage<MaterialOutboundVO> pages = materialOutboundService.selectMaterialOutboundPage(Condition.getPage(query), materialOutbound);
		return R.data(pages);
	}

	/**
	 * 出库明细 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入materialOutbound")
	public R save(@Valid @RequestBody MaterialOutboundEntity materialOutbound) {
		return R.status(materialOutboundService.save(materialOutbound));
	}

	/**
	 * 出库明细 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入materialOutbound")
	public R update(@Valid @RequestBody MaterialOutboundEntity materialOutbound) {
		return R.status(materialOutboundService.updateById(materialOutbound));
	}

	/**
	 * 出库明细 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "新增或修改", description  = "传入materialOutbound")
	public R submit(@Valid @RequestBody MaterialOutboundEntity materialOutbound) {
		return R.status(materialOutboundService.saveOrUpdate(materialOutbound));
	}

	/**
	 * 出库明细 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "删除", description  = "传入ids")
	public R remove(@Parameter(name = "主键集合", required = true) @RequestParam String ids) {
		return R.status(materialOutboundService.removeByIds(Func.toLongList(ids)));
	}


	/**
	 * 导出数据
	 */
	@GetMapping("/export-materialOutbound")
	@ApiOperationSupport(order = 9)
	@Operation(summary = "导出数据", description  = "传入materialOutbound")
	public void exportMaterialOutbound(@Parameter(hidden = true) @RequestParam Map<String, Object> materialOutbound, BladeUser bladeUser, HttpServletResponse response) {
		QueryWrapper<MaterialOutboundEntity> queryWrapper = Condition.getQueryWrapper(materialOutbound, MaterialOutboundEntity.class);
		//if (!AuthUtil.isAdministrator()) {
		//	queryWrapper.lambda().eq(MaterialOutbound::getTenantId, bladeUser.getTenantId());
		//}
		queryWrapper.lambda().eq(MaterialOutboundEntity::getIsDeleted, BladeConstant.DB_NOT_DELETED);
		List<MaterialOutboundExcel> list = materialOutboundService.exportMaterialOutbound(queryWrapper);
		ExcelUtil.export(response, "出库明细数据" + DateUtil.time(), "出库明细数据表", list, MaterialOutboundExcel.class);
	}

}
