/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.material.excel;


import lombok.Data;

import java.lang.Double;
import java.util.Date;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import com.alibaba.excel.annotation.write.style.ContentRowHeight;
import com.alibaba.excel.annotation.write.style.HeadRowHeight;
import java.io.Serializable;
import java.io.Serial;


/**
 * 运维商库存明细 Excel实体类
 *
 * @author AlexWang
 * @since 2024-09-04
 */
@Data
@ColumnWidth(25)
@HeadRowHeight(20)
@ContentRowHeight(18)
public class MaterialOperationOutboundExcel implements Serializable {

	@Serial
	private static final long serialVersionUID = 1L;

	/**
	 * 公司物料id
	 */
	@ColumnWidth(20)
	@ExcelProperty("公司物料id")
	private Long materialId;
	/**
	 * 所属公司
	 */
	@ColumnWidth(20)
	@ExcelProperty("所属公司")
	private String companyId;
	/**
	 * 
	 */
	@ColumnWidth(20)
	@ExcelProperty("")
	private String operationId;
	/**
	 * 运维上名称
	 */
	@ColumnWidth(20)
	@ExcelProperty("运维上名称")
	private String operationName;
	/**
	 * 物料编码
	 */
	@ColumnWidth(20)
	@ExcelProperty("物料编码")
	private String code;
	/**
	 * 物料名称
	 */
	@ColumnWidth(20)
	@ExcelProperty("物料名称")
	private String name;
	/**
	 * 物料类型
	 */
	@ColumnWidth(20)
	@ExcelProperty("物料类型")
	private String type;
	/**
	 * 物料型号
	 */
	@ColumnWidth(20)
	@ExcelProperty("物料型号")
	private String model;
	/**
	 * 功率(KW)
	 */
	@ColumnWidth(20)
	@ExcelProperty("功率(KW)")
	private String power;
	/**
	 * 
	 */
	@ColumnWidth(20)
	@ExcelProperty("")
	private Double costPrice;
	/**
	 * 零售价
	 */
	@ColumnWidth(20)
	@ExcelProperty("零售价")
	private Double retailPrice;
	/**
	 * 物料描述
	 */
	@ColumnWidth(20)
	@ExcelProperty("物料描述")
	private String description;
	/**
	 * 设备sn码
	 */
	@ColumnWidth(20)
	@ExcelProperty("设备sn码")
	private String brand;
	/**
	 * 设备库存数量
	 */
	@ColumnWidth(20)
	@ExcelProperty("设备库存数量")
	private Integer stock;
	/**
	 * 冻结数量
	 */
	@ColumnWidth(20)
	@ExcelProperty("冻结数量")
	private Integer freezeAmount;
	/**
	 * 质保状态（1、在保 0过保）
	 */
	@ColumnWidth(20)
	@ExcelProperty("质保状态（1、在保 0过保）")
	private String unit;
	/**
	 * 是否已删除
	 */
	@ColumnWidth(20)
	@ExcelProperty("是否已删除")
	private Integer isDeleted;
	/**
	 * 明细类型
	 */
	@ColumnWidth(20)
	@ExcelProperty("明细类型")
	private String outType;
	/**
	 * 1进  2出
	 */
	@ColumnWidth(20)
	@ExcelProperty("1进  2出")
	private String outDirect;
	/**
	 * 明细数量
	 */
	@ColumnWidth(20)
	@ExcelProperty("明细数量")
	private Integer outCount;
	/**
	 * 出入库总额
	 */
	@ColumnWidth(20)
	@ExcelProperty("出入库总额")
	private Double outAmount;
	/**
	 * 审核人
	 */
	@ColumnWidth(20)
	@ExcelProperty("审核人")
	private Long allowUser;
	/**
	 * 审核时间
	 */
	@ColumnWidth(20)
	@ExcelProperty("审核时间")
	private Date allowTime;
	/**
	 * 关联单号id
	 */
	@ColumnWidth(20)
	@ExcelProperty("关联单号id")
	private String applyId;
	/**
	 * 
	 */
	@ColumnWidth(20)
	@ExcelProperty("")
	private String remarks;

}
