/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.nh.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.nh.pojo.entity.InverterDataEntity;
import org.springblade.modules.nh.pojo.vo.InverterDataVO;
import org.springblade.modules.nh.excel.InverterDataExcel;
import org.springblade.modules.nh.wrapper.InverterDataWrapper;
import org.springblade.modules.nh.service.IInverterDataService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 逆变器数据表 控制器
 *
 * @author AlexWang
 * @since 2024-05-31
 */
@RestController
@AllArgsConstructor
@RequestMapping("nh/inverterData")
@Tag(name = "逆变器数据表", description = "逆变器数据表接口")
public class InverterDataController extends BladeController {

	private final IInverterDataService inverterDataService;

	/**
	 * 逆变器数据表 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入inverterData")
	public R<InverterDataVO> detail(InverterDataEntity inverterData) {
		InverterDataEntity detail = inverterDataService.getOne(Condition.getQueryWrapper(inverterData));
		return R.data(InverterDataWrapper.build().entityVO(detail));
	}
	/**
	 * 逆变器数据表 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "分页", description  = "传入inverterData")
	public R<IPage<InverterDataVO>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> inverterData, Query query) {
		IPage<InverterDataEntity> pages = inverterDataService.page(Condition.getPage(query), Condition.getQueryWrapper(inverterData, InverterDataEntity.class));
		return R.data(InverterDataWrapper.build().pageVO(pages));
	}

	/**
	 * 逆变器数据表 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入inverterData")
	public R<IPage<InverterDataVO>> page(InverterDataVO inverterData, Query query) {
		IPage<InverterDataVO> pages = inverterDataService.selectInverterDataPage(Condition.getPage(query), inverterData);
		return R.data(pages);
	}

	/**
	 * 逆变器数据表 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入inverterData")
	public R save(@Valid @RequestBody InverterDataEntity inverterData) {
		return R.status(inverterDataService.save(inverterData));
	}

	/**
	 * 逆变器数据表 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入inverterData")
	public R update(@Valid @RequestBody InverterDataEntity inverterData) {
		return R.status(inverterDataService.updateById(inverterData));
	}

	/**
	 * 逆变器数据表 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "新增或修改", description  = "传入inverterData")
	public R submit(@Valid @RequestBody InverterDataEntity inverterData) {
		return R.status(inverterDataService.saveOrUpdate(inverterData));
	}

	/**
	 * 逆变器数据表 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "删除", description  = "传入ids")
	public R remove(@Parameter(name = "主键集合", required = true) @RequestParam String ids) {
		return R.status(inverterDataService.removeByIds(Func.toLongList(ids)));
	}

}
