/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.nh.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.nh.pojo.entity.MonitorEntryStatEntity;
import org.springblade.modules.nh.pojo.vo.MonitorEntryStatVO;
import org.springblade.modules.nh.excel.MonitorEntryStatExcel;
import org.springblade.modules.nh.wrapper.MonitorEntryStatWrapper;
import org.springblade.modules.nh.service.IMonitorEntryStatService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 外部电站同步 控制器
 *
 * @author AlexWang
 * @since 2024-06-01
 */
@RestController
@AllArgsConstructor
@RequestMapping("nh/monitorEntryStat")
@Tag(name = "外部电站同步", description = "外部电站同步接口")
public class MonitorEntryStatController extends BladeController {

	private final IMonitorEntryStatService monitorEntryStatService;

	/**
	 * 外部电站同步 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入monitorEntryStat")
	public R<MonitorEntryStatVO> detail(MonitorEntryStatEntity monitorEntryStat) {
		MonitorEntryStatEntity detail = monitorEntryStatService.getOne(Condition.getQueryWrapper(monitorEntryStat));
		return R.data(MonitorEntryStatWrapper.build().entityVO(detail));
	}
	/**
	 * 外部电站同步 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "分页", description  = "传入monitorEntryStat")
	public R<IPage<MonitorEntryStatVO>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> monitorEntryStat, Query query) {
		IPage<MonitorEntryStatEntity> pages = monitorEntryStatService.page(Condition.getPage(query), Condition.getQueryWrapper(monitorEntryStat, MonitorEntryStatEntity.class));
		return R.data(MonitorEntryStatWrapper.build().pageVO(pages));
	}

	/**
	 * 外部电站同步 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入monitorEntryStat")
	public R<IPage<MonitorEntryStatVO>> page(MonitorEntryStatVO monitorEntryStat, Query query) {
		IPage<MonitorEntryStatVO> pages = monitorEntryStatService.selectMonitorEntryStatPage(Condition.getPage(query), monitorEntryStat);
		return R.data(pages);
	}

	/**
	 * 外部电站同步 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入monitorEntryStat")
	public R save(@Valid @RequestBody MonitorEntryStatEntity monitorEntryStat) {
		return R.status(monitorEntryStatService.save(monitorEntryStat));
	}

	/**
	 * 外部电站同步 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入monitorEntryStat")
	public R update(@Valid @RequestBody MonitorEntryStatEntity monitorEntryStat) {
		return R.status(monitorEntryStatService.updateById(monitorEntryStat));
	}

	/**
	 * 外部电站同步 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "新增或修改", description  = "传入monitorEntryStat")
	public R submit(@Valid @RequestBody MonitorEntryStatEntity monitorEntryStat) {
		return R.status(monitorEntryStatService.saveOrUpdate(monitorEntryStat));
	}

	/**
	 * 外部电站同步 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "删除", description  = "传入ids")
	public R remove(@Parameter(name = "主键集合", required = true) @RequestParam String ids) {
		return R.status(monitorEntryStatService.removeByIds(Func.toLongList(ids)));
	}
}
