/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.nh.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.nh.pojo.entity.MonitorInverterDailyStatEntity;
import org.springblade.modules.nh.pojo.vo.MonitorInverterDailyStatVO;
import org.springblade.modules.nh.wrapper.MonitorInverterDailyStatWrapper;
import org.springblade.modules.nh.service.IMonitorInverterDailyStatService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 逆变器日数据 控制器
 *
 * @author AlexWang
 * @since 2024-06-07
 */
@RestController
@AllArgsConstructor
@RequestMapping("nh/monitorInverterDailyStat")
@Tag(name = "逆变器日数据", description = "逆变器日数据接口")
public class MonitorInverterDailyStatController extends BladeController {

	private final IMonitorInverterDailyStatService monitorInverterDailyStatService;

	/**
	 * 逆变器日数据 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入monitorInverterDailyStat")
	public R<MonitorInverterDailyStatVO> detail(MonitorInverterDailyStatEntity monitorInverterDailyStat) {
		MonitorInverterDailyStatEntity detail = monitorInverterDailyStatService.getOne(Condition.getQueryWrapper(monitorInverterDailyStat));
		return R.data(MonitorInverterDailyStatWrapper.build().entityVO(detail));
	}
	/**
	 * 逆变器日数据 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "分页", description  = "传入monitorInverterDailyStat")
	public R<IPage<MonitorInverterDailyStatVO>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> monitorInverterDailyStat, Query query) {
		IPage<MonitorInverterDailyStatEntity> pages = monitorInverterDailyStatService.page(Condition.getPage(query), Condition.getQueryWrapper(monitorInverterDailyStat, MonitorInverterDailyStatEntity.class));
		return R.data(MonitorInverterDailyStatWrapper.build().pageVO(pages));
	}

	/**
	 * 逆变器日数据 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入monitorInverterDailyStat")
	public R<IPage<MonitorInverterDailyStatVO>> page(MonitorInverterDailyStatVO monitorInverterDailyStat, Query query) {
		IPage<MonitorInverterDailyStatVO> pages = monitorInverterDailyStatService.selectMonitorInverterDailyStatPage(Condition.getPage(query), monitorInverterDailyStat);
		return R.data(pages);
	}

	/**
	 * 逆变器日数据 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入monitorInverterDailyStat")
	public R save(@Valid @RequestBody MonitorInverterDailyStatEntity monitorInverterDailyStat) {
		return R.status(monitorInverterDailyStatService.save(monitorInverterDailyStat));
	}

	/**
	 * 逆变器日数据 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入monitorInverterDailyStat")
	public R update(@Valid @RequestBody MonitorInverterDailyStatEntity monitorInverterDailyStat) {
		return R.status(monitorInverterDailyStatService.updateById(monitorInverterDailyStat));
	}

	/**
	 * 逆变器日数据 新增或修改
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "新增或修改", description  = "传入monitorInverterDailyStat")
	public R submit(@Valid @RequestBody MonitorInverterDailyStatEntity monitorInverterDailyStat) {
		return R.status(monitorInverterDailyStatService.saveOrUpdate(monitorInverterDailyStat));
	}

	/**
	 * 逆变器日数据 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "删除", description  = "传入ids")
	public R remove(@Parameter(name = "主键集合", required = true) @RequestParam String ids) {
		return R.status(monitorInverterDailyStatService.removeByIds(Func.toLongList(ids)));
	}

}
