/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.nh.excel;


import lombok.Data;

import java.math.BigDecimal;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import com.alibaba.excel.annotation.write.style.ContentRowHeight;
import com.alibaba.excel.annotation.write.style.HeadRowHeight;
import java.io.Serializable;
import java.io.Serial;


/**
 * 逆变器数据表 Excel实体类
 *
 * @author AlexWang
 * @since 2024-05-31
 */
@Data
@ColumnWidth(25)
@HeadRowHeight(20)
@ContentRowHeight(18)
public class InverterDataExcel implements Serializable {

	@Serial
	private static final long serialVersionUID = 1L;

	/**
	 * 设备ID
	 */
	@ColumnWidth(20)
	@ExcelProperty("设备ID")
	private Long deviceId;
	/**
	 * 输出功率
	 */
	@ColumnWidth(20)
	@ExcelProperty("输出功率")
	private BigDecimal pac;
	/**
	 * 发电小时数
	 */
	@ColumnWidth(20)
	@ExcelProperty("发电小时数")
	private BigDecimal pwHour;
	/**
	 * 实际安装功率
	 */
	@ColumnWidth(20)
	@ExcelProperty("实际安装功率")
	private BigDecimal realPower;
	/**
	 * 装机容量
	 */
	@ColumnWidth(20)
	@ExcelProperty("装机容量")
	private BigDecimal power;
	/**
	 * 当日发电量
	 */
	@ColumnWidth(20)
	@ExcelProperty("当日发电量")
	private BigDecimal etd;
	/**
	 * 当月发电量
	 */
	@ColumnWidth(20)
	@ExcelProperty("当月发电量")
	private BigDecimal etm;
	/**
	 * 累计发电量
	 */
	@ColumnWidth(20)
	@ExcelProperty("累计发电量")
	private BigDecimal eto;
	/**
	 * 当年发电量
	 */
	@ColumnWidth(20)
	@ExcelProperty("当年发电量")
	private BigDecimal ety;
	/**
	 * 直流发电功率
	 */
	@ColumnWidth(20)
	@ExcelProperty("直流发电功率")
	private BigDecimal dcPac;
	/**
	 * 采集器SN
	 */
	@ColumnWidth(20)
	@ExcelProperty("采集器SN")
	private String collectorSn;
	/**
	 * 逆变器无功功率
	 */
	@ColumnWidth(20)
	@ExcelProperty("逆变器无功功率")
	private BigDecimal reactivePower;
	/**
	 * 逆变器无功功率单位
	 */
	@ColumnWidth(20)
	@ExcelProperty("逆变器无功功率单位")
	private String reactivePowerStr;
	/**
	 * 逆变器视在功率
	 */
	@ColumnWidth(20)
	@ExcelProperty("逆变器视在功率")
	private BigDecimal apparentPower;
	/**
	 * 逆变器视在功率单位
	 */
	@ColumnWidth(20)
	@ExcelProperty("逆变器视在功率单位")
	private String apparentPowerStr;
	/**
	 * 电网频率hz
	 */
	@ColumnWidth(20)
	@ExcelProperty("电网频率hz")
	private BigDecimal fac;
	/**
	 * pv直流电压存贮格式如下（220:10|220:10）
	 */
	@ColumnWidth(20)
	@ExcelProperty("pv直流电压存贮格式如下（220:10|220:10）")
	private String pvModules;
	/**
	 * mppt存贮格式如下（220:10|220:10）
	 */
	@ColumnWidth(20)
	@ExcelProperty("mppt存贮格式如下（220:10|220:10）")
	private String mpptModules;
	/**
	 * uvw(220:10:55|220:10:55|220:10:55)
	 */
	@ColumnWidth(20)
	@ExcelProperty("uvw(220:10:55|220:10:55|220:10:55)")
	private String uvwModules;
	/**
	 * 温度
	 */
	@ColumnWidth(20)
	@ExcelProperty("温度")
	private BigDecimal temperature;

}
