/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.nh.excel;


import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Data;

import java.math.BigDecimal;
import java.util.Date;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import com.alibaba.excel.annotation.write.style.ContentRowHeight;
import com.alibaba.excel.annotation.write.style.HeadRowHeight;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.io.Serial;


/**
 * 电站 Excel实体类
 *
 * @author AlexWang
 * @since 2024-05-31
 */
@Data
@ColumnWidth(25)
@HeadRowHeight(20)
@ContentRowHeight(18)
public class PowerStationExcel {

	/**
	 * 电站编号
	 */
	@ColumnWidth(20)
	@ExcelProperty("电站状态")
	private String statusStr;

	/**
	 * 电站编号
	 */
	@ColumnWidth(20)
	@ExcelProperty("电站编号")
	private String stationCode;

	/*@ColumnWidth(20)
	@ExcelProperty("电站名称")
	private String stationName;*/


	@ColumnWidth(20)
	@ExcelProperty("运维单号")
	private String orderNumber;

	@ColumnWidth(20)
	@ExcelProperty("品牌")
	private String brand;

	/**
	 * 业主名称
	 */
	@ColumnWidth(20)
	@ExcelProperty("业主名称")
	private String ownerName;

	@ColumnWidth(16)
	@ExcelProperty("省")
	private String province;

	@ColumnWidth(16)
	@ExcelProperty("市")
	private String city;

	@ColumnWidth(16)
	@ExcelProperty("区")
	private String district;

	/**
	 * 详细地址
	 */
	@ColumnWidth(30)
	@ExcelProperty("详细地址")
	private String address;


	/**
	 * 实际安装容量
	 */
	@ColumnWidth(20)
	@ExcelProperty("安装容量KW")
	private Double actualCapacity;

	/**
	 * 当期新增发电量
	 */
	@ColumnWidth(20)
	@ExcelProperty("日电量(KWh)")
	private Double etd;
	/**
	 * 当期新增发电量
	 */
	@ColumnWidth(20)
	@ExcelProperty("月电量(KWh)")
	private Double etm;
	/**
	 * 当期新增发电量
	 */
	@ColumnWidth(20)
	@ExcelProperty("年电量(KWh)")
	private Double ety;
	/**
	 * 累计发电量
	 */
	@ColumnWidth(20)
	@ExcelProperty("累计电量(KWh)")
	private Double eto;

	@ColumnWidth(20)
	@ExcelProperty("逆变器SN")
	private String sn;
	/**
	 * 进件序号
	 */
	@ColumnWidth(20)
	@ExcelProperty("进件序号")
	private String applyNo;

	@ColumnWidth(20)
	@ExcelProperty("资方")
	private String capitalName;

	/**
	 * 运维服务商名称
	 */
	@ColumnWidth(20)
	@ExcelProperty("代理商")
	private String agentName;

	@ColumnWidth(20)
	@ExcelProperty("投放通过时间")
	private String paymentTime;

	@ColumnWidth(20)
	@ExcelProperty("质保到期时间")
	private String warrantyExpiresTime;

	/**
	 * 运维服务商名称
	 */
	@ColumnWidth(20)
	@ExcelProperty("运维服务商名称")
	private String omProviderName;



	@ColumnWidth(20)
	@ExcelProperty("首次发电时间")
	private String fisGenTime;

	/**
	 * 异常时长
	 */
	@ColumnWidth(20)
	@ExcelProperty("异常时长")
	private String errDuring;

	/**
	 * 离线时长
	 */
	@ColumnWidth(20)
	@ExcelProperty("离线时长")
	private String lastDuringStr;

	public void initDefaultData() {
		if(this.etm==null){
			this.etm = 0d;
		}
		if(this.etd==null){
			this.etd = 0d;
		}
		if(this.ety==null){
			this.ety = 0d;
		}
		if(this.eto==null){
			this.eto = 0d;
		}
		if(this.actualCapacity==null){
			this.actualCapacity = 0d;
		}
	}

}
