/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.nh.pojo.dto.open;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springblade.modules.nh.pojo.entity.PowerStationEntity;

import java.io.Serial;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 电站 数据传输对象实体类
 *
 * @author AlexWang
 * @since 2024-05-31
 */
@Data
public class YXPowerStationDTO {
	@Serial
	private static final long serialVersionUID = 1L;

	private Long id;

	/**
	 * 电站编号
	 */
	@Schema(description = "电站编号")
	private String stationCode;

	/**
	 * 电站名称
	 */
	@Schema(description = "电站名称")
	private String stationName;

	/**
	 * 业主名称
	 */
	@Schema(description = "业主名称")
	private String ownerName;
	/**
	 * 业主电话
	 */
	@Schema(description = "业主电话")
	private String ownerPhone;
	/**
	 * sn码序列号(多个用逗号拼接)
	 */
	@Schema(description = "sn码序列号(多个用逗号拼接)")
	private String sn;
	/**
	 * 省
	 */
	@Schema(description = "省")
	private String province;
	/**
	 * 市
	 */
	@Schema(description = "市")
	private String city;
	/**
	 * 区
	 */
//	@Schema(description = "区")
//	private String district;
	/**
	 * 详细地址
	 */
	@Schema(description = "详细地址")
	private String address;

	/**
	 * 实际安装容量
	 */
	@Schema(description = "实际安装容量")
	private BigDecimal actualCapacity;
	/**
	 * 完工日期
	 */
	@Schema(description = "完工日期")
	private Date installAcceptTime;
	/**
	 * 首日发电日期
	 */
	@Schema(description = "首日发电日期")
	private Date startGenDate;

	@Schema(description = "首日并网日期")
	private Date mergeTime;
	/**
	 * 进件序号
	 */
	@Schema(description = "进件序号")
	private String applyNo;
	/**
	 * 日发电
	 */
	private Double etd;
	/**
	 * 月发发电
	 */
	private Double etm;
	/**
	 * 年发电
	 */
	private Double ety;
	/**
	 * 累计发电
	 */
	private Double eta;

	/**
	 * 逆变器ID集合，用，分开
	 */
	private String  nbqSn;

	public void init(){
		this.etd= 0d;
		this.etm = 0d;
		this.ety= 0d;
		this.ety = 0d;
		this.actualCapacity= new BigDecimal(0);
	}

}
