/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.operation.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.operation.pojo.entity.OperationTechnicalDataEntity;
import org.springblade.modules.operation.pojo.vo.OperationTechnicalDataVO;
import org.springblade.modules.operation.excel.OperationTechnicalDataExcel;
import org.springblade.modules.operation.wrapper.OperationTechnicalDataWrapper;
import org.springblade.modules.operation.service.IOperationTechnicalDataService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 技术资料表 控制器
 *
 * @author BladeX
 * @since 2024-06-26
 */
@RestController
@AllArgsConstructor
@RequestMapping("sf/operationTechnicalData")
@Tag(name = "技术资料表", description = "技术资料表接口")
public class OperationTechnicalDataController extends BladeController {

	private final IOperationTechnicalDataService operationTechnicalDataService;

	/**
	 * 技术资料表 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入operationTechnicalData")
	public R<OperationTechnicalDataVO> detail(OperationTechnicalDataEntity operationTechnicalData) {
		OperationTechnicalDataEntity detail = operationTechnicalDataService.getOne(Condition.getQueryWrapper(operationTechnicalData));
		return R.data(OperationTechnicalDataWrapper.build().entityVO(detail));
	}
	/**
	 * 技术资料表 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "分页", description  = "传入operationTechnicalData")
	public R<IPage<OperationTechnicalDataVO>> list(@Parameter(hidden = true) @RequestParam Map<String, Object> operationTechnicalData, Query query) {
		operationTechnicalData.put("status","0");
		IPage<OperationTechnicalDataEntity> pages = operationTechnicalDataService.page(Condition.getPage(query), Condition.getQueryWrapper(operationTechnicalData, OperationTechnicalDataEntity.class).orderByDesc("create_time"));
		return R.data(OperationTechnicalDataWrapper.build().pageVO(pages));
	}

	@GetMapping("/page")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "管理员分页", description  = "传入operationTechnicalData")
	public R<IPage<OperationTechnicalDataVO>> page(OperationTechnicalDataVO technicalDataVO, Query query) {
		IPage<OperationTechnicalDataVO> pages = operationTechnicalDataService.selectOperationTechnicalDataPage(Condition.getPage(query), technicalDataVO);
		return R.data(pages);
	}

	/**
	 * 技术资料表 新增
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@Operation(summary = "新增", description  = "传入operationTechnicalData")
	public R save(@Valid @RequestBody OperationTechnicalDataEntity operationTechnicalData) {
		return R.status(operationTechnicalDataService.addTechnical(operationTechnicalData));
	}

	/**
	 * 技术资料表 修改
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@Operation(summary = "修改", description  = "传入operationTechnicalData")
	public R update(@Valid @RequestBody OperationTechnicalDataEntity operationTechnicalData) {
		return R.status(operationTechnicalDataService.updateTechnical(operationTechnicalData));
	}

	/**
	 * 技术资料表 删除
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@Operation(summary = "逻辑删除", description  = "传入ids")
	public R remove(@Parameter(name = "主键集合", required = true) @RequestParam String ids) {
		return R.status(operationTechnicalDataService.deleteTechnical(ids));
	}


	/**
	 * 导出数据
	 */
	@GetMapping("/export-operationTechnicalData")
	@ApiOperationSupport(order = 9)
	@Operation(summary = "导出数据", description  = "传入operationTechnicalData")
	public void exportOperationTechnicalData(@Parameter(hidden = true) @RequestParam Map<String, Object> operationTechnicalData, BladeUser bladeUser, HttpServletResponse response) {
		QueryWrapper<OperationTechnicalDataEntity> queryWrapper = Condition.getQueryWrapper(operationTechnicalData, OperationTechnicalDataEntity.class);
		queryWrapper.lambda().eq(OperationTechnicalDataEntity::getStatus, BladeConstant.DB_NOT_DELETED);
		List<OperationTechnicalDataExcel> list = operationTechnicalDataService.exportOperationTechnicalData(queryWrapper);
		ExcelUtil.export(response, "技术资料表数据" + DateUtil.time(), "技术资料表数据表", list, OperationTechnicalDataExcel.class);
	}

}
