/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.operation.controller;

import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.operation.pojo.entity.PlatformEmployeeEntity;
import org.springblade.modules.operation.pojo.vo.PlatformEmployeeVO;
import org.springblade.modules.operation.excel.PlatformEmployeeExcel;
import org.springblade.modules.operation.wrapper.PlatformEmployeeWrapper;
import org.springblade.modules.operation.service.IPlatformEmployeeService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 平台员工权限表 控制器
 *
 * @author BladeX
 * @since 2024-06-27
 */
@RestController
@AllArgsConstructor
@RequestMapping("sf/platformEmployee")
@Tag(name = "平台员工权限表", description = "平台员工权限表接口")
public class PlatformEmployeeController extends BladeController {

	private final IPlatformEmployeeService platformEmployeeService;

	/**
	 * 平台员工权限表 详情
	 */
	@GetMapping("/empDetail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入platformEmployee")
	public R<PlatformEmployeeVO> empDetail(@Parameter(name = "userId",description = "主键id",required = true) @RequestParam String userId) {
		PlatformEmployeeEntity detail = platformEmployeeService.empDetail(userId);
		return R.data(PlatformEmployeeWrapper.build().entityVO(detail));
	}

	/**
	 * 编辑  员工权限
	 */
	@PostMapping("/editeEmpAuth")
	@ApiOperationSupport(order = 6)
	@Operation(summary = "编辑员工权限", description  = "传入platformEmployee")
	public R editeEmpAuth(@Valid @RequestBody PlatformEmployeeEntity platformEmployee) {
		return R.status(platformEmployeeService.editeEmpAuth(platformEmployee));
	}


}
