/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.operation.pojo.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import jakarta.validation.constraints.NotEmpty;
import lombok.Data;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.EqualsAndHashCode;
import org.springblade.core.mp.base.BaseEntity;
import org.springblade.core.tenant.mp.TenantEntity;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 网点基础信息表 实体类
 *
 * @author lzp
 * @since 2024-06-13
 */
@Data
@TableName("sf_operation_site")
@Schema(description = "OperationSite对象")
public class OperationSiteEntity implements Serializable {

	@Schema(description = "主键")
	@TableId(value = "id", type = IdType.ASSIGN_UUID)
	private String id;
	@Schema(description = "对应user表记录id")
	private Long userId;
	@Schema(description = "服务商编号，即唯一码：以录入时间生成编号")
	private String number;
	@Schema(description = "服务商名称", requiredMode = Schema.RequiredMode.REQUIRED)
	private String name;
	@Schema(description = "联系人", requiredMode = Schema.RequiredMode.REQUIRED)
	private String contacts;
	@Schema(description = "企业法人")
	private String corporator;
	@NotEmpty(message = "省不能为空")
	@Schema(description = "省", requiredMode = Schema.RequiredMode.REQUIRED)
	private String province;
	@Schema(description = "市", requiredMode = Schema.RequiredMode.REQUIRED)
	@NotEmpty(message = "市不能为空")
	private String city;
	@Schema(description = "区县", requiredMode = Schema.RequiredMode.REQUIRED)
	@NotEmpty(message = "区县不能为空")
	private String area;
	@Schema(description = "省代码", requiredMode = Schema.RequiredMode.REQUIRED)
	private String provinceCode;
	@Schema(description = "市代码", requiredMode = Schema.RequiredMode.REQUIRED)
	private String cityCode;
	@Schema(description = "区县代码", requiredMode = Schema.RequiredMode.REQUIRED)
	private String areaCode;
	@Schema(description = "详细地址", requiredMode = Schema.RequiredMode.REQUIRED)
	private String address;
	@Schema(description = "纬度，经度")
	private String lnglat;
	@Schema(description = "手机号码", requiredMode = Schema.RequiredMode.REQUIRED)
	private String mobile;
	@Schema(description = "营业执照号")
	private String licenseNumber;
	@Schema(description = "营业执照图片地址、相关资质")
	private String licenseImg;
	@Schema(description = "状态(0：启用 1：删除；2：未授权；3.禁用；4暂未合作)")
	private String status;
	@Schema(description = "审核标示 0.注册默认值（未完善资料）  1.完善资料  3特殊服务商标记（暂时只可手动设置）")
	private String checkFlag;
	@Schema(description = "备注")
	private String remarks;
	@Schema(description = "创建人user_id")
	private Long createBy;
	@Schema(description = "创建时间")
	private Date createTime;
	@Schema(description = "更新时间")
	private Date updateTime;
	@Schema(description = "可提现金额")
	private BigDecimal withdrawableCash;
	@Schema(description = "钱包总额")
	private BigDecimal totalWallet;
	@Schema(description = "服务商拒单次数")
	private Integer refusalCount;
	/**
	 * 保证金金额
	 */
	@Schema(description = "保证金金额")
	private BigDecimal earnestMoney;
	/**
	 * 缴纳保证金 1.是 2.否 默认否
	 */
	@Schema(description = "缴纳保证金 1.是 2.否 默认否")
	private String payTheDeposit;
	/**
	 * 结算方式 1.单次结算 2.包月结算
	 */
	@Schema(description = "结算方式 1.单次结算 2.包月结算")
	private String setType;



}
