/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.order.pojo.vo;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.NullSerializer;
import io.swagger.v3.oas.annotations.media.Schema;
import org.springblade.modules.order.pojo.dto.OperationOrderSettleDTO;
import org.springblade.modules.order.pojo.entity.OperationOrderEntity;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springblade.modules.order.pojo.entity.OperationOrderFeedbackEntity;
import org.springblade.modules.order.utils.Target;

import java.io.Serial;
import java.math.BigDecimal;
import java.util.List;

/**
 * 工单基础信息表 视图实体类
 *
 * @author lzp
 * @since 2024-06-18
 */
@Data
@EqualsAndHashCode(callSuper = true)
public class OperationOrderVO extends OperationOrderEntity {
	@Serial
	private static final long serialVersionUID = 1L;
	@Schema(description = "过程信息")
	private List<Target> guochengList;

	@Schema(description = "服务措施名称")
	private String serviceMeasuresName;

	@Schema(description = "服务类型名称")
	private String serviceTypeName;


	private OperationOrderFeedbackEntity feedbackEntity;
	@Schema(description = "反馈图片")
	private List<String> feedImgs;

	@Schema(description = "完工时间差")
	private String disEndTime;

	@Schema(description = "是否超期 1是 0否( 统计的是是否超出质保期的电站)")
	private String ifOverDue;

	@Schema(description = "费用结算信息")
	private OperationOrderSettleDTO orderSettleDTO;

	@Schema(description = "告警名称")
	private String alarmName;

	@Schema(description = "结算金额")
	@JsonSerialize(nullsUsing = NullSerializer.class)
	private BigDecimal settleAmount;

	@Schema(description = "扣款金额")
	@JsonSerialize(nullsUsing = NullSerializer.class)
	private BigDecimal deductionAmount;

	@Schema(description = "实际支付金额")
	@JsonSerialize(nullsUsing = NullSerializer.class)
	private BigDecimal actualAmount;

}
