/**
 * BladeX Commercial License Agreement
 * Copyright (c) 2018-2099, https://bladex.cn. All rights reserved.
 * <p>
 * Use of this software is governed by the Commercial License Agreement
 * obtained after purchasing a license from BladeX.
 * <p>
 * 1. This software is for development use only under a valid license
 * from BladeX.
 * <p>
 * 2. Redistribution of this software's source code to any third party
 * without a commercial license is strictly prohibited.
 * <p>
 * 3. Licensees may copyright their own code but cannot use segments
 * from this software for such purposes. Copyright of this software
 * remains with BladeX.
 * <p>
 * Using this software signifies agreement to this License, and the software
 * must not be used for illegal purposes.
 * <p>
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY. The author is
 * not liable for any claims arising from secondary or illegal development.
 * <p>
 * Author: Chill Zhuang (bladejava@qq.com)
 */
package org.springblade.modules.partner.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import jakarta.validation.Valid;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.annotation.PreAuth;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.nh.pojo.entity.AlarmCodeEntity;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.partner.pojo.entity.PartnerEntity;
import org.springblade.modules.partner.pojo.vo.PartnerVO;
import org.springblade.modules.partner.excel.PartnerExcel;
import org.springblade.modules.partner.wrapper.PartnerWrapper;
import org.springblade.modules.partner.service.IPartnerService;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.excel.util.ExcelUtil;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.constant.RoleConstant;
import java.util.Map;
import java.util.List;
import jakarta.servlet.http.HttpServletResponse;

/**
 * 合作商基础信息表 控制器
 *
 * @author lzp
 * @since 2024-06-18
 */
@RestController
@AllArgsConstructor
@RequestMapping("sf/partner")
@Tag(name = "合作商基础信息表", description = "合作商基础信息表接口")
public class PartnerController extends BladeController {

	private final IPartnerService partnerService;

	/**
	 * 合作商基础信息表 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@Operation(summary = "详情", description  = "传入partner")
	public R<PartnerVO> detail(PartnerEntity partner) {
		PartnerEntity detail = partnerService.getOne(Condition.getQueryWrapper(partner));
		return R.data(PartnerWrapper.build().entityVO(detail));
	}
	/**
	 * 合作商基础信息表 分页
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@Operation(summary = "不分页", description  = "传入partner")
	public R<List<PartnerVO>> list() {
		List<PartnerVO> pages = partnerService.listPartner();
		return R.data(pages);
	}

	/**
	 * 合作商基础信息表 自定义分页
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@Operation(summary = "分页", description  = "传入partner")
	public R<IPage<PartnerVO>> page(PartnerVO partner, Query query) {
		IPage<PartnerVO> pages = partnerService.selectPartnerPage(Condition.getPage(query), partner);
		return R.data(pages);
	}



}
